'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _util = require('./util');

var _util2 = _interopRequireDefault(_util);

var _cache = require('./cache');

var _cache2 = _interopRequireDefault(_cache);

var _loader = require('./loader');

var _loader2 = _interopRequireDefault(_loader);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var currentPath = _util2.default.currentDir;
var modulesPath = _path2.default.join(currentPath, 'node_modules' + _path2.default.sep);

var npmPath = void 0,
    src = void 0,
    dist = void 0;

exports.default = {
  getPkgConfig(lib) {
    var pkg = _util2.default.readFile(_path2.default.join(modulesPath, lib, 'package.json'));
    try {
      pkg = JSON.parse(pkg);
    } catch (e) {
      pkg = null;
    }
    return pkg;
  },
  resolveDeps(code, type, opath) {
    var _this = this;

    return code.replace(/require\(['"]([\w\d_\-\.\/@]+)['"]\)/ig, function (match, lib) {
      var resolved = lib;
      var target = '',
          source = '',
          ext = '',
          needCopy = false;

      if (lib[0] === '.') {
        // require('./something'');
        source = _path2.default.join(opath.dir, lib); // e:/src/util
        if (type === 'npm') {
          target = _path2.default.join(npmPath, _path2.default.relative(modulesPath, source));
          needCopy = true;
        } else {
          // e:/dist/util
          target = _util2.default.getDistPath(source);
          needCopy = false;
        }
      } else if (lib.indexOf('/') === -1 || // require('asset')
      lib.indexOf('/') === lib.length - 1 || // reqiore('a/b/something/')
      lib[0] === '@' && lib.indexOf('/') !== -1 && lib.lastIndexOf('/') === lib.indexOf('/') // require('@abc/something')
      ) {
          var pkg = _this.getPkgConfig(lib);
          if (!pkg) {
            throw Error('找不到模块: ' + lib);
          }
          var main = pkg.main || 'index.js';
          if (pkg.browser && typeof pkg.browser === 'string') {
            main = pkg.browser;
          }
          source = _path2.default.join(modulesPath, lib, main);
          target = _path2.default.join(npmPath, lib, main);
          lib += _path2.default.sep + main;
          ext = '';
          needCopy = true;
        } else {
        // require('babel-runtime/regenerator')
        source = _path2.default.join(modulesPath, lib);
        target = _path2.default.join(npmPath, lib);
        ext = '';
        needCopy = true;
      }

      if (_util2.default.isFile(source + '.js')) {
        ext = '.js';
      } else if (_util2.default.isFile(source + '.js')) {
        ext = '.js';
      } else if (_util2.default.isDir(source) && _util2.default.isFile(source + _path2.default.sep + 'index.js')) {
        ext = _path2.default.sep + 'index.js';
      } else if (_util2.default.isFile(source)) {
        ext = '';
      } else {
        throw '找不到文件: ' + source;
      }
      source += ext;
      target += ext;
      lib += ext;
      resolved = lib;

      if (needCopy) {
        if (!_cache2.default.checkBuildCache(source)) {
          _cache2.default.setBuildCache(source);
          _util2.default.log('依赖: ' + _path2.default.relative(process.cwd(), target), '拷贝');
          /*let dirname = path.dirname(target)
          mkdirp.sync(dirname)*/

          _this.compile('js', null, 'npm', _path2.default.parse(source));
        }
      }
      if (type === 'npm') {
        if (lib[0] !== '.') {
          resolved = _path2.default.join('..' + _path2.default.sep, _path2.default.relative(opath.dir, modulesPath), lib);
        } else {
          if (lib[0] === '.' && lib[1] === '.') resolved = './' + resolved;
        }
      } else {
        resolved = _path2.default.relative(_util2.default.getDistPath(opath, opath.ext, src, dist), target);
      }
      resolved = resolved.replace(/\\/g, '/').replace(/^\.\.\//, './');
      return `require('${resolved}')`;
    });
  },
  npmHack(filename, code) {
    // 一些库（redux等） 可能会依赖 process.env.NODE_ENV 进行逻辑判断
    // 这里在编译这一步直接做替换 否则报错
    code = code.replace(/process\.env\.NODE_ENV/g, process.env.NODE_ENV);
    switch (filename) {
      case 'lodash.js':
      case '_global.js':
        code = code.replace(/Function(\s*\'\s*return this\s*\'\s*)()/, 'this');
        break;
      case '_html.js':
        code = 'module.exports = false;';
        break;
      case '_microtask.js':
        code = code.replace(/if\s*\(\s*Observer\s*\)/, 'if(false && Observer)');
        // IOS 1.10.2 Promise BUG
        code = code.replace(/Promise\s*&&\s*Promise\.resolve/, 'false && Promise && Promise.resolve');
        break;
      case '_freeGlobal.js':
        code = code.replace(/module\.exports\s*=\s*freeGlobal/, 'module.exports = freeGlobal || this');
    }
    return code;
  },
  compile(lang, code, type, opath) {
    var _this2 = this;

    var config = _util2.default.getConfig();
    src = _cache2.default.getSrc();
    dist = _cache2.default.getDist();
    npmPath = _path2.default.join(currentPath, dist, 'npm' + _path2.default.sep);
    if (!code) {
      code = _util2.default.readFile(_path2.default.join(opath.dir, opath.base));
      if (code === null) {
        throw '打开文件失败: ' + _path2.default.join(opath.dir, opath.base);
      }
    }

    var compiler = _loader2.default.loadCompiler(lang);

    if (!compiler) {
      return;
    }
    compiler(code, config.compilers[lang] || {}).then(function (compileResult) {
      var sourceMap = void 0;
      if (typeof compileResult === 'string') {
        code = compileResult;
      } else {
        sourceMap = compileResult.map;
        code = compileResult.code;
      }
      if (type !== 'npm') {
        if (type === 'page' || type === 'app') {
          code = code.replace(/exports\.default\s*=\s*(\w+)/ig, function (m, defaultExport) {
            if (defaultExport === 'undefined') {
              return '';
            }

            if (type === 'page') {
              var pagePath = _path2.default.join(_path2.default.relative(_cache2.default.getAppPath(), opath.dir), opath.name).replace(/\\/ig, '/');
              return `\nPage(require('wemix').default.$createPage(${defaultExport} , '${pagePath}'))\n`;
            } else {
              var appConfig = (0, _stringify2.default)(config.appConfig || {});
              return `\nApp(require('wemix').default.$createApp(${defaultExport}, ${appConfig}))\n`;
            }
          });
        }
      }

      //转换 foobar instanceof Function 为 typeof foobar ==='function'
      //由于微信重定义了全局的Function对象，所以moment等npm库会出现异常
      code = code.replace(/([\w\[\]a-d\.]+)\s*instanceof Function/g, function (matchs, word) {
        return ' typeof ' + word + " ==='function' ";
      });

      code = _this2.resolveDeps(code, type, opath);

      var target = void 0;
      if (type !== 'npm') {
        target = _util2.default.getDistPath(opath, 'js');
      } else {
        code = _this2.npmHack(opath.base, code);
        target = _path2.default.join(npmPath, _path2.default.relative(modulesPath, _path2.default.join(opath.dir, opath.base)));
      }

      if (sourceMap) {
        sourceMap.sources = [opath.name + '.js'];
        sourceMap.file = opath.name + '.js';
        var Base64 = require('js-base64').Base64;
        code += `\r\n//# sourceMappingURL=data:application/json;charset=utf-8;base64,${Base64.encode((0, _stringify2.default)(sourceMap || {}))}`;
      }

      var plg = new _loader2.default.PluginHelper(config.plugins, {
        type: type,
        code: code,
        file: target,
        output(p) {
          _util2.default.output(p.action, p.file);
        },
        done(result) {
          _util2.default.output('写入', result.file);
          _util2.default.writeFile(target, result.code);
        }
      });
      // 缓存文件修改时间戳
      _cache2.default.saveBuildCache();
    }).catch(function (e) {
      console.log(e);
    });
  }
};
//# sourceMappingURL=compile-script.js.map