'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _postcss = require('postcss');

var _postcss2 = _interopRequireDefault(_postcss);

var _cssnano = require('cssnano');

var _cssnano2 = _interopRequireDefault(_cssnano);

var _autoprefixer = require('autoprefixer');

var _autoprefixer2 = _interopRequireDefault(_autoprefixer);

var _util = require('./util');

var _util2 = _interopRequireDefault(_util);

var _cache = require('./cache');

var _cache2 = _interopRequireDefault(_cache);

var _loader = require('./loader');

var _loader2 = _interopRequireDefault(_loader);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var LANG_MAP = {
  'less': '.less'
};

exports.default = {
  compile() {
    var style = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'css';
    var content = arguments[1];
    var opath = arguments[2];
    var components = arguments[3];

    var config = _util2.default.getConfig();
    var src = _cache2.default.getSrc();
    var dist = _cache2.default.getDist();
    var filepath = _path2.default.join(opath.dir, opath.name + '.less');
    var target = _util2.default.getDistPath(opath, 'wxss', src, dist);

    var options = (0, _assign2.default)({}, config.compilers[style] || {});

    if (!content) {
      content = _util2.default.readFile(filepath) || '';
    }

    var compiler = _loader2.default.loadCompiler(style);
    if (!compiler) {
      throw `未发现相关 ${lang} 编译器配置，请联系开发人员。`;
    }
    var imports = [];
    var hasImport = true;
    while (hasImport) {
      var length = 0;
      var _import = content.substring(content.indexOf('@import'), content.indexOf('.less') + 5);
      if (_import.indexOf('@import') > -1 && _import.indexOf('.less') > -1) {
        length = _import.length;
        var close = content.substr(content.indexOf('.less') + 5, 1);
        if (close === '\'' || close === '\"') {
          _import += close;
          length++;
          var end = content.substr(content.indexOf('.less') + 6, 1);
          if (end === ';') {
            length++;
            _import += end;
          }
        }
        content = content.substr(length + 1);
        imports.push(_import.replace('.less', '.wxss'));
      } else {
        hasImport = false;
      }
    }
    var compress = options.compress;
    delete options.compress;

    var references = [];
    if (components && !_util2.default.isEmpty(components)) {
      (0, _keys2.default)(components).forEach(function (com) {
        if (components[com].indexOf('.js') > -1) {
          references.push(`@import '${components[com].replace('.js', '.wxss')}';`);
        } else {
          references.push(`@import '${components[com]}.wxss';`);
        }
      });
    }

    compiler(content, options, filepath).then(function (css) {
      css = references.join('\n') + (references.length ? '\n' + css : css);
      css = imports.join('\n') + (imports.length ? '\n' + css : css);
      var plg = new _loader2.default.PluginHelper(config.plugins, {
        type: 'css',
        code: css,
        file: target,
        output(p) {
          _util2.default.output(p.action, p.file);
        },
        done(rst) {
          _util2.default.output('写入', rst.file);
          var cssPlugins = [_autoprefixer2.default];
          if (compress) {
            cssPlugins.push((0, _cssnano2.default)());
          }
          (0, _postcss2.default)(cssPlugins).process(rst.code).then(function (result) {
            _util2.default.writeFile(target, result.css);
          });
        }
      });
    });
  }
};
//# sourceMappingURL=compile-style.js.map