'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _xmldom = require('xmldom');

var _cache = require('./cache');

var _cache2 = _interopRequireDefault(_cache);

var _util = require('./util');

var _util2 = _interopRequireDefault(_util);

var _compileConfig = require('./compile-config');

var _compileConfig2 = _interopRequireDefault(_compileConfig);

var _compileHtml = require('./compile-html');

var _compileHtml2 = _interopRequireDefault(_compileHtml);

var _compileStyle = require('./compile-style');

var _compileStyle2 = _interopRequireDefault(_compileStyle);

var _compileScript = require('./compile-script');

var _compileScript2 = _interopRequireDefault(_compileScript);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = {
  createParser() {
    return new _xmldom.DOMParser({ errorHandler: {
        warning(x) {
          if (x.indexOf('missed value!!') > -1) {
            // ignore warnings
          } else _util2.default.warning(x);
        },
        error(x) {
          _util2.default.error(x);
        }
      } });
  },
  grabConfigFromScript(str, n) {
    var stash = [],
        rst = '';
    for (var i = n, l = str.length; i < l; i++) {
      if (str[i] === '{') stash.push('{');
      if (str[i] === '}') {
        stash.pop();
        if (stash.length === 0) {
          rst += '}';
          break;
        }
      }
      if (stash.length) {
        rst += str[i];
      }
    }
    return rst;
  },
  resolveWemix(xml, opath) {
    var _this = this;

    var config = _util2.default.getConfig();
    var filepath = void 0;
    if (typeof xml === 'object' && xml.dir) {
      opath = xml;
      filepath = _path2.default.join(xml.dir, xml.base);
    } else {
      opath = _path2.default.parse(xml);
      filepath = xml;
    }
    var content = _util2.default.readFile(filepath);
    if (content === null) {
      _util2.default.error('打开文件失败: ' + filepath);
      return null;
    }
    if (content === '') {
      return null;
    }
    var moduleId = _util2.default.genId(filepath);
    var rst = {
      moduleId: moduleId,
      template: {
        code: _util2.default.readFile(_path2.default.join(opath.dir, opath.name + '.html')) || '<view></view>',
        src: _path2.default.join(opath.dir, opath.name + '.html'),
        type: 'wxml'
      },
      script: {
        code: content,
        src: _path2.default.join(opath.dir, opath.base),
        type: 'babel'
      },
      style: {
        code: _util2.default.readFile(_path2.default.join(opath.dir, opath.name + '.less')) || 'Page {}',
        src: _path2.default.join(opath.dir, opath.name + '.less'),
        type: 'wxss'
      }
    };

    // get config
    (function () {
      var match = rst.script.code.match(/[\s\r\n]this.config\s*=[\s\r\n]*/);
      match = match ? match[0] : undefined;
      rst.config = match ? _this.grabConfigFromScript(rst.script.code, rst.script.code.indexOf(match) + match.length) : false;
      try {
        if (rst.config) {
          rst.config = new Function(`return ${rst.config}`)();
        }
      } catch (e) {
        _util2.default.output('错误', _path2.default.join(opath.dir, opath.base));
        _util2.default.error(`解析config出错，报错信息：${e}\r\n${rst.config}`);
      }
    })();

    // get imports
    (function () {
      var coms = {};
      rst.script.code.replace(/import\s*([\w\-\_]*)\s*from\s*['"]([\w\-\_\.\/]*)['"]/ig, function (match, com, path) {
        coms[com] = path;
      });

      var matchData = rst.script.code.match(/[\s\r\n]this.data\s*=[\s\r\n]*/);
      matchData = matchData ? matchData[0] : undefined;
      if (matchData) {
        var defaultData = matchData ? _this.grabConfigFromScript(rst.script.code, rst.script.code.indexOf(matchData) + matchData.length) : false;
        rst.script.code = rst.script.code.replace(/[\s\r\n]this.data\s*=[\s\r\n]*/, function (match, item, index) {
          return `this.$data = ${defaultData}\r\n${match}`;
        });
      }

      var matchCustomerData = rst.script.code.match(/[\s\r\n]this.customerData\s*=[\s\r\n]*/);
      matchCustomerData = matchCustomerData ? matchCustomerData[0] : undefined;
      if (matchCustomerData) {
        var defaultCustomerData = matchCustomerData ? _this.grabConfigFromScript(rst.script.code, rst.script.code.indexOf(matchCustomerData) + matchCustomerData.length) : false;
        rst.script.code = rst.script.code.replace(/[\s\r\n]this.customerData\s*=[\s\r\n]*/, function (match, item, index) {
          return `this.$customerData = ${defaultCustomerData}\r\n${match}`;
        });
      }

      var matchProps = rst.script.code.match(/[\s\r\n]this.defaultProps\s*=[\s\r\n]*/);
      matchProps = matchProps ? matchProps[0] : undefined;
      var defaultProps = matchProps ? _this.grabConfigFromScript(rst.script.code, rst.script.code.indexOf(matchProps) + matchProps.length) : false;

      var matchComponents = rst.script.code.match(/[\s\r\n]this.components\s*=[\s\r\n]*/);
      matchComponents = matchComponents ? matchComponents[0] : undefined;
      var components = matchComponents ? _this.grabConfigFromScript(rst.script.code, rst.script.code.indexOf(matchComponents) + matchComponents.length) : false;
      var vars = (0, _keys2.default)(coms).map(function (com, i) {
        return `var ${com} = "${coms[com]}"`;
      }).join('\r\n');

      try {
        if (components) {
          rst.template.components = new Function(`${vars}\r\nreturn ${components}`)();
          rst.style.components = new Function(`${vars}\r\nreturn ${components}`)();
        } else {
          rst.template.components = {};
          rst.style.components = {};
        }
        if (defaultProps) {
          rst.template.defaultProps = new Function(`return ${defaultProps}`)();
        } else {
          rst.template.defaultProps = {};
        }
      } catch (e) {
        _util2.default.output('错误', _path2.default.join(opath.dir, opath.base));
        _util2.default.error(`解析components出错，报错信息：${e}\r\n${vars}\r\nreturn ${components}`);
      }
    })();
    return rst;
  },
  remove(opath, ext) {
    var src = _cache2.default.getSrc();
    var dist = _cache2.default.getDist();
    ext = ext || opath.substr(1);
    var target = _util2.default.getDistPath(opath, ext, src, dist);
    if (_util2.default.isFile(target)) {
      _util2.default.log('配置: ' + _path2.default.relative(_util2.default.currentDir, target), '删除');
      _fs2.default.unlinkSync(target);
    }
  },
  compile(opath) {
    var filepath = _path2.default.join(opath.dir, opath.base);
    var src = _cache2.default.getSrc();
    var dist = _cache2.default.getDist();
    var pages = _cache2.default.getPages();

    var type = '';
    var relative = _path2.default.relative(_util2.default.currentDir, filepath);
    if (filepath === _path2.default.join(_util2.default.currentDir, src, 'app.js')) {
      type = 'app';
      _util2.default.log('入口: ' + relative, '编译');
      _cache2.default.setAppPath(_path2.default.join(_util2.default.currentDir, src));
    } else if (pages.indexOf(relative) > -1) {
      type = 'page';
      _util2.default.log('页面: ' + relative, '编译');
    } else if (relative.indexOf(_path2.default.sep + 'components' + _path2.default.sep) > -1) {
      type = 'component';
      _util2.default.log('组件: ' + relative, '编译');
    } else {
      _util2.default.log('Other: ' + relative, '编译');
    }

    var wmix = this.resolveWemix(opath);

    if (!wmix) {
      return;
    }

    if (type === 'app') {
      // 第一个编译
      var _pages = wmix.config.pages.map(function (v) {
        return _path2.default.join(src, v + '.js');
      });
      wmix.config.subPackages && wmix.config.subPackages.forEach(function (pack) {
        pack && pack.pages.forEach(function (page) {
          _pages.push(_path2.default.join(src, pack.root, page + '.js'));
        });
      });
      _cache2.default.setPages(_pages);
      delete wmix.template;
    } else if (type === 'component') {
      delete wmix.config;
    }

    if (wmix.template && wmix.template.code && type !== 'component') {
      // App 和 Component 不编译 wxml
      var pe = _compileHtml2.default.compile(wmix.template);
      wmix.script.code = wmix.script.code.replace(/[\s\r\n]this.components\s*=[\s\r\n]*/, function (match, item, index) {
        return `this.$props = ${(0, _stringify2.default)(pe.props || {})}\r\nthis.$events = ${(0, _stringify2.default)(pe.events || {})}\r\n${match}`;
      });
    }

    _compileStyle2.default.compile('less', wmix.style.code, opath, wmix.style.components);

    if (wmix.config) {
      _compileConfig2.default.compile(wmix.config, opath);
    } else {
      this.remove(opath, 'json');
    }

    if (wmix.script.code) {
      _compileScript2.default.compile(wmix.script.type, wmix.script.code, type, opath);
    }
  }
};
//# sourceMappingURL=compile-wemix.js.map