'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _chokidar = require('chokidar');

var _chokidar2 = _interopRequireDefault(_chokidar);

var _compareVersions = require('compare-versions');

var _compareVersions2 = _interopRequireDefault(_compareVersions);

var _ignore = require('ignore');

var _ignore2 = _interopRequireDefault(_ignore);

var _cache = require('./cache');

var _cache2 = _interopRequireDefault(_cache);

var _util = require('./util');

var _util2 = _interopRequireDefault(_util);

var _loader = require('./loader');

var _loader2 = _interopRequireDefault(_loader);

var _compileStyle = require('./compile-style');

var _compileStyle2 = _interopRequireDefault(_compileStyle);

var _compileScript = require('./compile-script');

var _compileScript2 = _interopRequireDefault(_compileScript);

var _compileWemix = require('./compile-wemix');

var _compileWemix2 = _interopRequireDefault(_compileWemix);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var watchReady = false;
var preventDup = {};

exports.default = {
  watch(config) {
    var _this = this;

    config.watch = false;

    var wemixConfig = _util2.default.getConfig();
    var src = config.source || wemixConfig.src || 'src';
    var dist = config.output || wemixConfig.output || 'dist';

    _chokidar2.default.watch(`.${_path2.default.sep}${src}`, {
      depth: 99
    }).on('all', function (evt, filepath) {
      if ((evt === 'change' || evt === 'add') && watchReady && !preventDup[filepath]) {
        preventDup[filepath] = evt;
        config.file = _path2.default.join('..', filepath);
        _util2.default.log('文件: ' + filepath, '变更');
        _this.build(config, true);
        setTimeout(function () {
          preventDup[filepath] = false;
        }, 500);
      }
    }).on('ready', function () {
      watchReady = true;
      _util2.default.log('开始监听文件改动。', '信息');
    });
  },
  loadCompiler(compilers) {
    for (var k in compilers) {
      _loader2.default.loadCompiler(k);
    }
  },
  loadPlugins() {
    var plugins = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    _loader2.default.loadPlugin(plugins);
  },
  build(config, update) {
    var _this2 = this;

    var production = config.production;
    var wemixConfig = _util2.default.getConfig(production);
    this.loadCompiler(wemixConfig.compilers);
    this.loadPlugins(wemixConfig.plugins);
    var src = 'src';
    var dist = 'dist';
    config.source = src;
    config.dist = dist;
    var current = process.cwd();
    var files = _util2.default.getFiles(src);
    _cache2.default.setParams(config);
    _cache2.default.setSrc(src);
    _cache2.default.setDist(dist);

    // If dist/npm/wemix is not exsit, then clear the build cache.
    if (!_util2.default.isDir(_path2.default.join(_util2.default.currentDir, dist, 'npm', 'wemix'))) {
      config.cache = false;
    }
    if (!config.cache || config.production) {
      _cache2.default.clearBuildCache();
    }
    if (files.some(function (v) {
      return v === 'app.js';
    })) {
      // 如果编译文件包含app.js，且第一个不是 app.js
      if (_util2.default.isFile(_path2.default.join(current, src, 'app.js'))) {
        // src/app.js 存在, 则把它放在第一位, 因为后面需要取页面路径
        var newFiles = ['app.js'].concat(files.filter(function (v) {
          return v !== 'app.js';
        }));
        files = newFiles;
      } else {
        _util2.default.error('根目录不存在app.js');
      }
    }
    var pageFiles = [];
    files.forEach(function (f) {
      if (/^pages.*\.js$/.test(f)) {
        pageFiles.push(_path2.default.parse(_path2.default.join(current, src, f)));
      }
    });

    files.forEach(function (f) {
      var opath = _path2.default.parse(_path2.default.join(current, src, f));
      _this2.compile(opath, pageFiles, update);
    });

    if (config.watch) {
      this.watch(config);
    }
  },
  compile(opath, pageFiles, update) {
    if (update && opath.dir.indexOf('/src/components') > -1 && _util2.default.isFile(_path2.default.join(opath.dir, opath.name + '.html')) && !_cache2.default.checkBuildCache(_path2.default.join(opath.dir, opath.name + '.html'))) {
      pageFiles.forEach(function (pagePath) {
        _compileWemix2.default.compile(pagePath);
      });
    }
    if (opath.dir.indexOf('/src/components') > -1 || opath.dir.indexOf('/src/pages') > -1 || opath.base === 'app.js') {
      if (_util2.default.isFile(_path2.default.join(opath.dir, opath.name + '.html')) && !_cache2.default.checkBuildCache(_path2.default.join(opath.dir, opath.name + '.html')) || _util2.default.isFile(_path2.default.join(opath.dir, opath.name + '.less')) && !_cache2.default.checkBuildCache(_path2.default.join(opath.dir, opath.name + '.less')) || _util2.default.isFile(_path2.default.join(opath.dir, opath.name + '.js')) && !_cache2.default.checkBuildCache(_path2.default.join(opath.dir, opath.name + '.js'))) {
        _cache2.default.setBuildCache(_path2.default.join(opath.dir, opath.base));
        if (opath.ext === '.less' || opath.ext === '.html') {
          opath.base = opath.base.replace(/(.less|.html)/, '.js');
          opath.ext = '.js';
        }
      } else {
        return;
      }
    }
    var src = _cache2.default.getSrc();
    var dist = _cache2.default.getDist();
    var wemixConfig = _util2.default.getConfig();
    if (!_util2.default.isFile(opath)) {
      _util2.default.error('文件不存在：' + _util2.default.getRelative(opath));
      return;
    }
    switch (opath.ext) {
      case '.js':
        if (opath.dir.indexOf('/src/components') > -1 || opath.dir.indexOf('/src/pages') > -1 || opath.name === 'app') {
          _compileWemix2.default.compile(opath);
        } else {
          if (!_cache2.default.checkBuildCache(_path2.default.join(opath.dir, opath.base))) {
            _cache2.default.setBuildCache(_path2.default.join(opath.dir, opath.base));
            _compileScript2.default.compile('babel', null, 'js', opath);
          }
        }
        break;
      case '.less':
        if (opath.dir.indexOf('/src/components') === -1 && opath.dir.indexOf('/src/pages') === -1 && opath.name !== 'app') {
          if (!_cache2.default.checkBuildCache(_path2.default.join(opath.dir, opath.base))) {
            _cache2.default.setBuildCache(_path2.default.join(opath.dir, opath.base));
            _compileStyle2.default.compile('less', null, opath);
          }
        }
        break;
      default:
        if (!_cache2.default.checkBuildCache(_path2.default.join(opath.dir, opath.base))) {
          _cache2.default.setBuildCache(_path2.default.join(opath.dir, opath.base));
          _util2.default.output('拷贝', _path2.default.join(opath.dir, opath.base));

          var plg = new _loader2.default.PluginHelper(wemixConfig.plugins, {
            type: opath.ext.substr(1),
            code: null,
            file: _path2.default.join(opath.dir, opath.base),
            output(p) {
              _util2.default.output(p.action, p.file);
            },
            done(rst) {
              if (rst.code) {
                var target = _util2.default.getDistPath(_path2.default.parse(rst.file));
                _util2.default.writeFile(target, rst.code);
              } else {
                _util2.default.copy(_path2.default.parse(rst.file));
              }
            },
            error(rst) {
              _util2.default.warning(rst.err);
              _util2.default.copy(_path2.default.parse(rst.file));
            }
          });
        }
    }
  }
};
//# sourceMappingURL=compile.js.map