'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _safe = require('colors/safe');

var _safe2 = _interopRequireDefault(_safe);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _child_process = require('child_process');

var _rimraf = require('rimraf');

var _rimraf2 = _interopRequireDefault(_rimraf);

var _hashSum = require('hash-sum');

var _hashSum2 = _interopRequireDefault(_hashSum);

var _cache = require('./cache');

var _cache2 = _interopRequireDefault(_cache);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

_safe2.default.enabled = true;

_safe2.default.setTheme({
  'info': 'grey',
  '变更': 'bgYellow',
  '删除': 'bgMagenta',
  '执行': 'blue',
  '压缩': 'blue',
  '信息': 'grey',
  '完成': 'green',
  '创建': 'green',
  '监听': 'magenta',
  '错误': 'red',
  '测试': 'red',
  '拷贝': 'yellow',
  '编译': 'blue',
  '写入': 'green'
});
var ID_CACHE = {};

var utils = {
  currentDir: process.cwd(),
  cliDir: __dirname,
  getConfig(production) {
    var config = _cache2.default.getConfig();
    if (config && typeof config !== 'boolean' || config && config.production === production) return config;

    var configFile = _path2.default.join(this.currentDir, _path2.default.sep, 'wemix.config.js');
    if (this.isFile(configFile)) {
      config = require(configFile);
    }
    config = {
      production: production,
      compilers: {
        less: {
          // zindex 会被重新更改 导致不同的组件引用会有层级问题， 看wemix-compiler-less
          compress: false
        },
        babel: {
          sourceMap: !production,
          'presets': [['env', config.env || {}]],
          'plugins': [['transform-runtime', config.transformRuntime || {
            'helpers': false,
            'polyfill': true,
            'regenerator': false,
            'moduleName': 'babel-runtime'
          }], ['transform-class-properties'], ['transform-object-rest-spread']]
        }
      },
      plugins: {}
    };
    if (production) {
      config.plugins = {
        filemin: {
          filter: /\.(wxml|xml|json)$/,
          config: {}
        },
        uglifyjs: {
          filter: /\.js$/,
          config: {
            toplevel: true
          }
        },
        imagemin: {
          filter: /\.(jpg|png|jpeg)$/,
          config: {
            jpg: {
              quality: 80
            },
            png: {
              quality: 80
            }
          }
        }
      };
    }
    _cache2.default.setConfig(config);
    return config;
  },
  isFunction(fn) {
    return typeof fn === 'function';
  },
  isString(obj) {
    return toString.call(obj) === '[object String]';
  },
  isObject(obj) {
    return toString.call(obj) === '[object Object]';
  },
  isNumber(obj) {
    return toString.call(obj) === '[object Number]';
  },
  isBoolean(obj) {
    return toString.call(obj) === '[object Boolean]';
  },
  isArray(obj) {
    return Array.isArray(obj);
  },
  isFile(p) {
    p = typeof p === 'object' ? _path2.default.join(p.dir, p.base) : p;
    if (!_fs2.default.existsSync(p)) {
      return false;
    }
    return _fs2.default.statSync(p).isFile();
  },
  isDir(p) {
    if (!_fs2.default.existsSync(p)) {
      return false;
    }
    return _fs2.default.statSync(p).isDirectory();
  },
  datetime() {
    var date = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : new Date();
    var format = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'HH:mm:ss';

    var fn = function fn(d) {
      return ('0' + d).slice(-2);
    };
    if (date && this.isString(date)) {
      date = new Date(Date.parse(date));
    }
    var formats = {
      YYYY: date.getFullYear(),
      MM: fn(date.getMonth() + 1),
      DD: fn(date.getDate()),
      HH: fn(date.getHours()),
      mm: fn(date.getMinutes()),
      ss: fn(date.getSeconds())
    };
    return format.replace(/([a-z])\1+/ig, function (a) {
      return formats[a] || a;
    });
  },
  error(msg) {
    this.log(msg, 'error', false);
  },
  warning(msg) {
    this.log(msg, 'warning', false);
  },
  log(msg, type) {
    var showTime = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;

    var dateTime = showTime ? _safe2.default.gray(`[${this.datetime()}] `) : '';
    if (this.isObject(msg) || this.isArray(msg)) {
      msg = (0, _stringify2.default)(msg || {});
    }
    if (type && this.isString(type)) {
      type = type.toUpperCase();
      if (type === 'ERROR') {
        console.error(_safe2.default.red('[Error] ' + msg));
      } else if (type === 'WARNING') {
        console.error(_safe2.default.yellow('[WARNING] ' + msg));
      } else {
        var fn = _safe2.default[type] ? _safe2.default[type] : _safe2.default['info'];
        console.log(dateTime + fn(`[${type}]`) + ' ' + msg);
      }
    } else {
      console.log(dateTime + msg);
    }
  },
  getVersion() {
    var filepath = _path2.default.resolve(__dirname, '../package.json');
    var version = JSON.parse(this.readFile(filepath)).version;
    return version;
  },
  readFile(p) {
    var rst = '';
    p = typeof p === 'object' ? _path2.default.join(p.dir, p.base) : p;
    try {
      rst = _fs2.default.readFileSync(p, 'utf-8');
    } catch (e) {
      rst = null;
    }
    return rst;
  },
  mkdir(name) {
    var rst = true;
    try {
      _fs2.default.mkdirSync(name);
    } catch (e) {
      rst = e;
    }
    return rst;
  },
  writeFile(p, data) {
    var opath = this.isString(p) ? _path2.default.parse(p) : p;
    if (!this.isDir(opath.dir)) {
      _mkdirp2.default.sync(opath.dir);
    }
    _fs2.default.writeFileSync(p, data);
  },
  getFiles() {
    var _this = this;

    var dir = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : process.cwd();
    var prefix = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';

    var cfiles = _cache2.default.getFileList(dir);
    if (cfiles) return cfiles;
    dir = _path2.default.normalize(dir);
    if (!_fs2.default.existsSync(dir)) {
      return [];
    }
    var files = _fs2.default.readdirSync(dir);
    var rst = [];
    files.forEach(function (item) {
      var filepath = dir + _path2.default.sep + item;
      var stat = _fs2.default.statSync(filepath);
      if (stat.isFile()) {
        rst.push(prefix + item);
      } else if (stat.isDirectory()) {
        rst = rst.concat(_this.getFiles(_path2.default.normalize(dir + _path2.default.sep + item), _path2.default.normalize(prefix + item + _path2.default.sep)));
      }
    });

    _cache2.default.setFileList(dir, rst);
    return rst;
  },
  getDistPath(opath, ext, src, dist) {
    var relative = void 0;
    src = src || _cache2.default.getSrc();
    dist = dist || _cache2.default.getDist();
    if (typeof opath === 'string') opath = _path2.default.parse(opath);
    ext = ext ? ext[0] === '.' ? ext : '.' + ext : opath.ext;
    // 第三组件
    // 相对目录以node_modules开始
    if (_path2.default.relative(this.currentDir, opath.dir).indexOf('node_modules') === 0) {
      // if (opath.dir.indexOf(`${path.sep}${src}${path.sep}`) === -1 && opath.dir.indexOf('node_modules') > 1) {
      relative = _path2.default.relative(_path2.default.join(this.currentDir, 'node_modules'), opath.dir);
      relative = _path2.default.join('npm', relative);
    } else {
      relative = _path2.default.relative(_path2.default.join(this.currentDir, src), opath.dir);
    }
    return _path2.default.join(this.currentDir, dist, relative, opath.name + ext);
  },
  getModifiedTime(p) {
    return this.isFile(p) ? +_fs2.default.statSync(p).mtime : 1;
  },
  unique(arr) {
    var tmp = {},
        out = [];
    arr.forEach(function (v) {
      if (!tmp[v]) {
        tmp[v] = 1;
        out.push(v);
      }
    });
    return out;
  },
  unlink(p) {
    var rst = '';
    p = typeof p === 'object' ? _path2.default.join(p.dir, p.base) : p;
    try {
      rst = _fs2.default.unlinkSync(p);
    } catch (e) {
      rst = null;
    }
    return rst;
  },
  clearDist(callback) {
    var _this2 = this;

    var proConf = this.readFile(_path2.default.join(this.currentDir, 'dist/project.config.json'));
    var jsConf = this.readFile(_path2.default.join(this.currentDir, 'dist/jsconfig.json'));
    (0, _rimraf2.default)(_path2.default.join(this.currentDir, 'dist'), function () {
      proConf && _this2.writeFile(_path2.default.join(_this2.currentDir, 'dist/project.config.json'), proConf);
      jsConf && _this2.writeFile(_path2.default.join(_this2.currentDir, 'dist/jsconfig.json'), jsConf);
      callback();
    });
  },
  exec(cmd, quite) {
    return new _promise2.default(function (resolve, reject) {
      var fcmd = (0, _child_process.exec)(cmd, function (err, stdout, stderr) {
        if (err) {
          reject(err);
        } else {
          resolve(stdout, stderr);
        }
      });
      fcmd.stdout.on('data', function (chunk) {
        !quite && process.stdout.write(chunk);
      });
      fcmd.stderr.on('data', function (chunk) {
        !quite && process.stdout.write(chunk);
      });
    });
  },
  output(type, file, flag) {
    if (!flag) {
      flag = file.substr(file.lastIndexOf('.') + 1).toUpperCase();
      if (flag.length < 4) {
        var i = 4 - flag.length;
        while (i--) {
          flag += ' ';
        }
      }
    }
    this.log(flag + ': ' + _path2.default.relative(this.currentDir, file), type);
  },
  genId(filepath) {
    if (!ID_CACHE[filepath]) {
      ID_CACHE[filepath] = '_' + (0, _hashSum2.default)(filepath).slice(1, 8);
    }
    return ID_CACHE[filepath];
  },
  getComId(elem) {
    var tagName = elem.nodeName;
    return tagName;
  },
  elemToArray(elems) {
    var rst = [];
    for (var i = 0, len = elems.$$length; i < len; i++) {
      rst.push(elems[i]);
    }
    return rst;
  },
  findComponent(com) {
    var src = '';
    if (com.indexOf(_path2.default.sep) !== -1) {
      if (this.isFile(com + '.js')) {
        src = com + '.js';
      }
    } else {
      var comPath = _path2.default.join(this.currentDir, 'node_modules', com);
      if (this.isDir(comPath)) {
        var pkg = this.readFile(_path2.default.join(comPath, 'package.json'));
        try {
          pkg = JSON.parse(pkg);
        } catch (e) {
          pkg = null;
        }
        src = _path2.default.join(comPath, pkg.main);
      } else {
        var _comPath = _path2.default.join(this.currentDir, _cache2.default.getSrc(), 'components', com);
        if (this.isFile(_comPath + '.js')) {
          src = _comPath + '.js';
        } else if (this.isFile(_comPath + '/index.js')) {
          src = _comPath + '/index.js';
        } else if (this.isFile(_comPath + '/' + com + '.js')) {
          src = _comPath + '/' + com + '.js';
        }
      }
    }
    return src;
  },
  copy(opath, ext, src, dist) {
    var target = this.getDistPath(opath, ext, src, dist);
    this.writeFile(target, this.readFile(_path2.default.join(opath.dir, opath.base)));
    var readable = _fs2.default.createReadStream(_path2.default.join(opath.dir, opath.base));
    var writable = _fs2.default.createWriteStream(target);
    readable.pipe(writable);
  },
  getRelative(opath) {
    return _path2.default.relative(this.currentDir, _path2.default.join(opath.dir, opath.base));
  },
  isNoValue(value) {
    if (typeof value === 'undefined' || typeof value === 'object' && !value) {
      return true;
    } else {
      return false;
    }
  },
  isEmpty(obj) {
    return (0, _keys2.default)(obj).length === 0;
  }
};

exports.default = utils;
//# sourceMappingURL=util.js.map