'use strict';

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _commander = require('commander');

var _commander2 = _interopRequireDefault(_commander);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _child_process = require('child_process');

var _util = require('./util');

var _util2 = _interopRequireDefault(_util);

var _compile = require('./compile');

var _compile2 = _interopRequireDefault(_compile);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var displayVersion = function displayVersion() {
  var version = _util2.default.getVersion();
  var chars = ['   __    ____  __  __  _  _   _ ', '( \\/\\/ )(____)(  \\/  )( )( \\_/ )', ' )    (  )__)  )    ( ( ) ) _ ( ', '(__/\\__)(____)( /\\/\\ )(_)(_/ \\_)', '                                         '].join('\n');
  console.log('\n v' + version + '\n');
  console.log(chars);
};

var generateProject = function generateProject(name, config) {
  var inPlace = !name || name === '.';
  if (inPlace) {
    name = process.cwd().split(_path2.default.sep).pop();
    _util2.default.log('使用当前目录：' + name);
  } else {
    _util2.default.log('目录：' + name, '创建');
    if (_util2.default.mkdir(name) !== true) {
      _util2.default.error('创建目录失败。');
      return;
    }
    process.chdir(name);
  }

  _util2.default.currentDir = process.cwd();

  var packagePath = _path2.default.join(_util2.default.currentDir, 'package.json');

  if (_util2.default.isFile(packagePath) || _util2.default.isDir(_path2.default.join(_util2.default.currentDir, 'src'))) {
    _util2.default.error('目录不为空, 请请勿重复初始化', '错误');
    return;
  }

  var tplDir = '../templates/empty';
  var template = _path2.default.join(_util2.default.cliDir, tplDir, _path2.default.sep);
  var dependencies = ['wemix'];

  var pkg = _path2.default.join(template, 'package.json');
  pkg = _util2.default.readFile(pkg);
  pkg = JSON.parse(pkg);
  pkg.name = name;
  _util2.default.writeFile(packagePath, (0, _stringify2.default)(pkg || {}));
  _util2.default.log('配置: ' + 'package.json', '写入');

  var files = _util2.default.getFiles(template).filter(function (file) {
    return file !== 'package.json';
  });

  var copyFn = function copyFn(sourcePath) {
    return function (file) {
      var target = _path2.default.join(_util2.default.currentDir, file);

      var fileContent = _util2.default.readFile(_path2.default.join(sourcePath, file));
      _util2.default.writeFile(target, fileContent);
      _util2.default.log('文件: ' + file, '拷贝');
    };
  };
  files.forEach(copyFn(template));

  var cmd = 'npm install --registry https://registry.npm.taobao.org';
  var cmdD = `npm install ${dependencies.join(' ')} --save --registry https://registry.npm.taobao.org`;
  _util2.default.log('执行命令: ' + cmd, '执行');
  _util2.default.log('可能需要几分钟, 请耐心等待...', '信息');
  _util2.default.exec(cmd).then(function (d) {
    return _util2.default.exec(cmdD);
  }).then(function (d) {
    _util2.default.log('安装依赖完成', '完成');
    var cmd = 'wemix build';
    _util2.default.log('执行命令: ' + cmd, '执行');
    _util2.default.log('可能需要几分钟, 请耐心等待...', '信息');

    _util2.default.exec(cmd).then(function (d) {
      _util2.default.log('代码编译完成', '完成');
      _util2.default.log('项目初始化完成, 可以开始使用小程序。', '完成');
    }).catch(function (e) {
      _util2.default.log('代码编译出错', '错误');
    });
  }).catch(function (e) {
    _util2.default.log('安装依赖出错', '错误');
  });
};

var generatePage = function generatePage(name, config) {
  if (!name) {
    return _util2.default.log('请输入文件路径');
  }
  var className = void 0;
  var exts = ['js', 'less', 'html'];
  name = name.replace(/(^\s*)|(\s*$)/g, '');
  var target = _path2.default.join(process.cwd(), name);

  var pathSplit = target.split(_path2.default.sep);
  if (/index/i.test(pathSplit[pathSplit.length - 1])) {
    if (/pages/i.test(pathSplit[pathSplit.length - 2])) {
      className = 'Index';
    } else {
      className = pathSplit[pathSplit.length - 2].replace(/^\S/, function (s) {
        return s.toUpperCase();
      });
    }
  } else {
    className = pathSplit[pathSplit.length - 1].replace(/^\S/, function (s) {
      return s.toUpperCase();
    });
  }

  exts.forEach(function (item) {
    var fileContent = _util2.default.readFile(_path2.default.join(_util2.default.cliDir, `../templates/page/page.${item}`));
    if (item === 'js') {
      fileContent = fileContent.replace('{{PAGE}}', className);
    }
    _util2.default.writeFile(`${target}.${item}`, fileContent);
    _util2.default.log(`文件: ${target}.${item}`, '拷贝');
  });
};

var generateComponent = function generateComponent(name, config) {
  if (!name) {
    return _util2.default.log('请输入文件路径');
  }
  var className = void 0;
  var exts = ['js', 'less', 'html'];
  name = name.replace(/(^\s*)|(\s*$)/g, '');
  var target = _path2.default.join(process.cwd(), name);

  var pathSplit = target.split(_path2.default.sep);
  if (/index/i.test(pathSplit[pathSplit.length - 1])) {
    if (/pages/i.test(pathSplit[pathSplit.length - 2])) {
      className = 'Index';
    } else {
      className = pathSplit[pathSplit.length - 2].replace(/^\S/, function (s) {
        return s.toUpperCase();
      });
    }
  } else {
    className = pathSplit[pathSplit.length - 1].replace(/^\S/, function (s) {
      return s.toUpperCase();
    });
  }

  exts.forEach(function (item) {
    var fileContent = _util2.default.readFile(_path2.default.join(_util2.default.cliDir, `../templates/component/component.${item}`));
    if (item === 'js') {
      fileContent = fileContent.replace('{{COMPONENT}}', className);
    }
    _util2.default.writeFile(`${target}.${item}`, fileContent);
    _util2.default.log(`文件: ${target}.${item}`, '拷贝');
  });
};

_commander2.default.usage('[command] <options ...>');
_commander2.default.option('-v, --version', '显示版本号', function () {
  displayVersion();
});
_commander2.default.option('-V', '显示版本号', function () {
  displayVersion();
});
_commander2.default.option('--no-cache', '对于引用到的文件，即使无改动也会再次编译');
_commander2.default.option('-w, --watch', '监听文件改动');
_commander2.default.option('-p, --production', '生成线上代码');

_commander2.default.command('build').description('编译项目').action(function (projectPath) {
  if (!_util2.default.isDir(_path2.default.join(_util2.default.currentDir, 'node_modules'))) {
    _util2.default.error('请先执行npm install安装所需依赖', '错误');
  } else {
    if (!_commander2.default.cache || _commander2.default.production) {
      _util2.default.clearDist(function () {
        _compile2.default.build(_commander2.default);
      });
    } else {
      _compile2.default.build(_commander2.default, true);
    }
  }
});

_commander2.default.command('new <projectName>').description('生成项目').action(function (name) {
  generateProject(name || 'wemixProject', _commander2.default);
});

_commander2.default.command('mkpage <pageName>').description('生成页面').action(function (name) {
  generatePage(name, _commander2.default);
});

_commander2.default.command('mkcomponent <componentName>').description('生成组件').action(function (name) {
  generateComponent(name, _commander2.default);
});

_commander2.default.parse(process.argv);
//# sourceMappingURL=wemix.js.map